#include "DLImage.h"
#include "DLDocument.h"
#include "DLPage.h"
#include "DLZone.h"
#include "DLTagList.h"
#include "DLDocID.h"
#include "DLDocIDBatchTest.h"
#include "DLTestManager.h"
#include "DLConst.h"
#include "DLxml.h"

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <math.h>

using namespace std;

#define MAX_PATH 260

DLDocIDBatchTest::DLDocIDBatchTest(char* name, int argc, char** argv) : DLTest(name, argc, argv)
{
	testName = name;
	dlArgc = argc;
	dlArgv = argv;
}

DLDocIDBatchTest::~DLDocIDBatchTest(void)
{
}

void DLDocIDBatchTest::showHelp()
{
	/* Show command line help information */
	printf( "DocID [-x] [-l] filename\n" );
	printf( "   -x  ---  Write classification results into an xml file for each input image.\n" );
	printf( "            It creates a new xml file if no associated xml file exists.\n" );
	printf( "   -l  ---  File containing the list of input images to execute\n" );
	printf( "   -h  ---  Show help at command line\n" );	
}

int DLDocIDBatchTest::go()
{
	char inputFile[MAX_PATH];
	char logFile[MAX_PATH];
	bool bFileList = false;
	bool bOutputXML = false;

	/* Parse command line arguments */
	if(dlArgc == 1)
	{
		showHelp();
		return 0;
	}
    
	for(int i=1; i<dlArgc; i++)
	{
		if(dlArgv[i][0] != '-') break;
		++i;
		switch(tolower(dlArgv[i-1][1]))
		{
			case 'x':
				bOutputXML = true;
				i--;
				break;
			case 'l':
				bFileList = true;
				i--;
				break;
			case 'o':
				strcpy( logFile, dlArgv[i] );
				break;
			case '?':
			case 'h':
				showHelp();
				return 0;
			default:
				printf( "Unable to interprete your command!\n" );
				showHelp();
				return -1;
		}
	}

	/* Show commmand line help panel if incorrect arguments are supplied */
	if( i != dlArgc-1 )
	{
		showHelp();
		return -1;
	}

	/* Grab the input file name specified at the command line */
	strcpy( inputFile, dlArgv[i] );

	vector <string> imageFileList;
	string imageFileName;

	/* Read in the list of images for processing if the list argument in the command line is supplied */
	if(bFileList)
	{
		ifstream infile(inputFile);
		cout << "Batch file - " << string(inputFile) << endl << endl;
		appendToTestLog("Batch file - " + string(inputFile) + "\n");
		if(infile)
		{
			while (!infile.eof())
			{
				imageFileName = "";
				infile >> imageFileName;
				if(imageFileName != "\n" && imageFileName != "")
					imageFileList.push_back(imageFileName);
			}
		}
		infile.close();
	}
	/* Process the single file if the list argument in the command line is empty */
	else
	{
		imageFileList.push_back(static_cast<string>(inputFile));
		cout << "Image file - " << string(inputFile) << endl;
		appendToTestLog("Image file - " + string(inputFile) + "\n");
	}

	DLDocID::DLImageContentType imageType;
	DLDocument::DLPagePtrIterator page_Iter;
	string srcImageFullPathName;
	char docCounter[5], pageCounter[5], zoneCounter[5];
	int imageIndex;

	/* Only need to initialize the recognizer classifier once */
	string modelfilename = "model.txt";
	DLDocID docIDClassifier(modelfilename, int(256), true);

	/* Run the image content recognition application from input document images */
	for(imageIndex = 0; imageIndex < int(imageFileList.size()); imageIndex++)
	{
		srcImageFullPathName = (string)imageFileList[imageIndex];
		DLImage docImage(srcImageFullPathName.c_str());

		sprintf(docCounter, "%d", imageIndex+1);
		sprintf(pageCounter, "%d", 1);
		string docID = "Document" + static_cast<string>(docCounter);
		string pageID = "Page" + static_cast<string>(pageCounter);

		DLDocument currentDoc;

		bool xmlFileFound = false;
		int fileExtPosition = int(srcImageFullPathName.find("."));
		int dirSepPosition = srcImageFullPathName.find_last_of(static_cast<string>(DL_DIR_SEPARATOR), fileExtPosition);
		string srcImageFileName, xmlFileName, xmlFilePath, xmlFullPathName;

		if(dirSepPosition != -1)	
		{
			srcImageFileName = srcImageFullPathName.substr(dirSepPosition);
			xmlFileName = srcImageFullPathName.substr(dirSepPosition, fileExtPosition);
			xmlFilePath = srcImageFullPathName.substr(0, dirSepPosition);
		}
		else 
		{
			srcImageFileName = srcImageFullPathName;
			xmlFileName = srcImageFullPathName.substr(0, fileExtPosition);
			xmlFilePath = "";
		}
		xmlFileName = xmlFileName + ".xml";
		xmlFullPathName = xmlFilePath + xmlFileName;

		ifstream xmlfile(xmlFileName.c_str());
		if(xmlfile)
		{
			/* Read in the xml file associated with the input document image if the xml file exists */
			xmlFileFound = true;
			cout << "Reading xml file - " << string(xmlFileName) << endl << endl;	
			DLxml xmlInput;
			currentDoc = xmlInput.dlReadFromXmlFile( xmlFileName, xmlFilePath );
			page_Iter = currentDoc.begin();

			if((*page_Iter)->dlHasZones())
			{
				/* iterate over all the zones and erase previous detection results */
				DLPage::DLZonePtrIterator zone_Iter1, zone_Iter2;
				for ( zone_Iter1 = (*page_Iter)->begin(); zone_Iter1 != (*page_Iter)->end(); )
				{
					DLTagList currentTagList = (*zone_Iter1)->dlGetAllTags();
					if(!currentTagList.dlIsTagListEmpty() && currentTagList.dlGetTag("gedi_type") == "DL_TEXTLINEGT" && 
						currentTagList.dlGetTag("id") == "ImageTypeIcon")
					{
						zone_Iter2 = zone_Iter1;
						zone_Iter2++;
						(*page_Iter)->dlDeleteZone(*zone_Iter1);
						zone_Iter1 = zone_Iter2;
					}
					else {
						zone_Iter1++;
					}
				}
			}
		}
		else
		{
			/* Create a single page document for storing detection information if there is no associated xml file */
			DLDocument* thisDoc = new DLDocument(docImage, pageID, docID);
			currentDoc = *thisDoc;
			delete thisDoc;
			page_Iter = currentDoc.begin();
		}
		xmlfile.close();

		cout << "Classifying " << srcImageFileName.c_str() << " ..." << endl;
		imageType = docIDClassifier.test( docImage );

		bool bDisplayIcon = false;
		int zoneOffset = 0;
		int imageDimension = min(docImage.dlGetWidth(), docImage.dlGetHeight());
		double imageDimensionExp = log(double(imageDimension))/log(2.0);
		int displayExp = min(8, int(imageDimensionExp));
		if(displayExp >= 5) {
			bDisplayIcon = true;
			zoneOffset = int(pow(2, displayExp-1));
		}

		DLZone* typeIcon = new DLZone("", zoneOffset, zoneOffset, zoneOffset, zoneOffset);
		typeIcon->dlSetTag("gedi_type", "DL_TEXTLINEGT");
		typeIcon->dlSetTag("id", "ImageTypeIcon");
		typeIcon->dlSetTag("offsets", "");
		typeIcon->dlSetTag("segmentation", "word");
		switch ( imageType )
		{
			case DLDocID::DL_DOCUMENT_IMAGE:
				(*page_Iter)->dlSetTag( "ImageType", "Document", true );
				typeIcon->dlSetTag("contents", "Document");
				cout << "Document" << endl;
				break;
			case DLDocID::DL_IMAGE_WITH_TEXT:
				(*page_Iter)->dlSetTag( "ImageType", "ImageWithText", true );
				typeIcon->dlSetTag("contents", "Image with Text");
				cout << "Image with text" << endl;
				break;
			default:
				(*page_Iter)->dlSetTag( "ImageType", "Non-Document", true );
				typeIcon->dlSetTag("contents", "Non-Document Image");
				cout << "Non-Document image" << endl;
		}
		(*page_Iter)->dlAppendZone(typeIcon);

		/* Update existing xml file or create a new xml file containing the detection results */
		if(bOutputXML)
		{
			DLxml xmlOutput(currentDoc, srcImageFileName);	
			xmlOutput.dlSaveToXmlFile(xmlFullPathName);
			if(xmlFileFound) {
				cout << string(xmlFullPathName) << " updated" << endl << endl;
			}
			else {
				cout << string(xmlFullPathName) << " created" << endl << endl;
			}
		}
	}

	cout << imageIndex << " images classfied" << endl;
	
	appendToTestLog("\n\n\n******************************************************\n");
	appendToTestLog("**Test Summary\n");
	appendToTestLog("******************************************************\n\n");

	return 0;

}



