#ifndef _DLDOCIDBATCHTEST_H
#define _DLDOCIDBATCHTEST_H

#include "DLTest.h"
#include "DLZone.h"
#include <string>

/**
 * \class DLDocIDBatchTest
 *
 * \brief DLDocIDBatchTest is a command line application that runs the three-class classification
 * that identifies document, image with text and non-document image on a list of input images
 * specified by the user.
 * Note: Prior to running the command line executable, please make sure that the trained model file
 * is included inthe same directory
 *
 * Its command line syntax is:
 * <p>DocID [-x] [-l] filename\n</p>
 *
 * <p>The list of command line arguments are as follows:<br>
 *  -x  ---  Write classification results into an xml file for each input image.<br>
 *           It creates a new xml file if no associated xml file exists<br>
 *  -l  ---  File containing the list of input images to execute<br>
 *  -h  ---  Show help at command line<br>
 * </p><br>
 */
class DLDocIDBatchTest : public DLTest
{
public:

	/**
    * Default DLDocIDBatchTest Constructor
	* \param argc An integer that contains the count of arguments that follow in argv. 
	* The argc parameter is always greater than or equal to 1. 
	* \param argv An array of null-terminated strings representing command-line arguments
	* entered by the user of the program. By convention, argv[0] is the command with which the
	* program is invoked, argv[1] is the first command-line argument, and so on, until argv[argc],
	* which is always NULL.
	* \param name A string given to the test.
    */
	DLDocIDBatchTest(char* name, int argc, char** argv);
    
	/**
	* Default DLDocIDBatchTest Destructor
	*/
	~DLDocIDBatchTest(void);

	/**
	* The Main function of DLDocIDBatchTest. Running the image content recognition driver application on a list of document images
	*/
	int go();

	/**
	* Show help information for running the image content recognition driver application
	*/
	void showHelp();

};

#endif
