#include "DLDocID.h"

using namespace std;

DLDocID::DLDocID(string modelfilename, int windowsize, bool test)
{
    model_filename = modelfilename;
	winsize = windowsize;
	if(test) {
		classifier = new DLSVMClassifier(model_filename, true);
	}
}

DLDocID::~DLDocID()
{
	delete classifier;
	classifier = NULL;
}

bool DLDocID::batchTraining( string filelistname )
{
	//read in the image file list.
	int cnt = 0;
	vector<string> filenames;
	vector<string> trainingVector;

	FILE* fp = fopen( filelistname.c_str(), "r" );
	fscanf(fp,"%d\n", &cnt);

	// process the image
	vector< vector<double> > data;
	vector<int> dataclass;

	for( int i = 0; i < cnt; i++)
	{
		char pimginputfilename[256];
		int iclass = 0;
		memset(pimginputfilename, 0 , 256);
		fscanf(fp,"%d\n", &iclass);
		fscanf(fp,"%s\n", pimginputfilename);
		string imginputfilename = pimginputfilename;
		string txtoutputfilename = imginputfilename + "_output_line.txt";


		DLImage im;
		im.dlLoadImage(imginputfilename.c_str());

		if( im.dlGetDepth() == DL_BITDEPTH_24 )
			im = DLBitsPerPixelConverter::dlDownscaleColor2Gray_global(im);
		else if( im.dlGetDepth() == DL_BITDEPTH_1 )
			im = DLBitsPerPixelConverter::dlBinary2Gray(im);

		DLLBP lbp;
		vector<double> freq = lbp.getNormalizedHistogram( im.dlResizeImage( winsize, winsize ) );

		data.push_back(freq);
		dataclass.push_back(iclass);
	}

	classifier = new DLSVMClassifier(model_filename.c_str());
	classifier->batchTraining( data, dataclass );

	return true;
}

DLDocID::DLImageContentType DLDocID::test( const DLImage& inputImage )
{
	DLImage im = inputImage;
	if( im.dlGetDepth() == DL_BITDEPTH_24 )
		im = DLBitsPerPixelConverter::dlDownscaleColor2Gray_global(im);
	else if( im.dlGetDepth() == DL_BITDEPTH_1 )
		im = DLBitsPerPixelConverter::dlBinary2Gray(im);

	// feature.
	DLLBP lbp;
	vector<double> freq = lbp.getNormalizedHistogram( im.dlResizeImage( winsize, winsize ) );

	// test	
	int iclass = classifier->test( freq );
	return DLImageContentType(iclass);
}

DLDocID::DLImageContentType DLDocID::test( string inputimagename )
{
	DLImage im(inputimagename.c_str());

	if( im.dlGetDepth() == DL_BITDEPTH_24 )
		im = DLBitsPerPixelConverter::dlDownscaleColor2Gray_global(im);
	else if( im.dlGetDepth() == DL_BITDEPTH_1 )
		im = DLBitsPerPixelConverter::dlBinary2Gray(im);

	// Use rotation invariant texture feature for analysis
	DLLBP lbp;
	vector<double> freq = lbp.getNormalizedHistogram( im.dlResizeImage( winsize, winsize ) );

	// test
	int iclass = classifier->test( freq );

	return DLImageContentType(iclass);
}
