#ifndef _DLDOCID_H_
#define _DLDOCID_H_

#include "DLLBP.h"
#include "DLImage.h"
#include "DLBitsPerPixelConverter.h"
#include "DLSVMClassifier.h"

#include <vector>

using namespace std;

/**
 * \brief The DLDocID object performs image content recognition for diverse imagery by texture analysis,
 * and classifies an image into one of the image content type defined by DLImageContentType.
 */
class DLDocID
{
public:

	/**
	*  Image content types
	*/
	enum DLImageContentType
	{
		DL_DOCUMENT_IMAGE = 1,
		DL_IMAGE_WITH_TEXT = 2,
		DL_NONDOCUMENT_IMAGE = 3
	};

	/**
	* DLDocID Constructor: Initialize parameters for training or testing
	* In training, a new model file in LIBSVM format will be created by the given file name.
	* In testing, the specified model file in LIBSVM format will be used for classification.
	* \param model_filename file name of the SVM model file in LIBSVM format.
	* \param windowsize dimension of the rescaled image size in which texture analysis is carried out
	* \param test Boolean type indicator. True for testing, false for training.
	*/
	DLDocID(string modelfilename, int windowsize, bool test);

	/**
	* Batch training using a list of training images
	* \param filelistname name of the file containing the list of training images
	* \return true if successful
	*/
	bool batchTraining( string filelistname );

	/**
	* Classify a test image of a given file name
	* \param testfilename file name of the test image
	* \return the predicted image content type defined in DLImageContentType
	*/
	DLImageContentType test( string testfilename );

	/**
	* Classify a test image of DLImage type
	* \param testImage file name of the test image
	* \return the predicted image content type defined in DLImageContentType
	*/
	DLImageContentType test( const DLImage& testImage );

protected:

	/// User a multi-class SVM classifier for classification
	DLSVMClassifier* classifier;

	/// File name of the classifier model file
	string model_filename;

	/// Window size of the rescaled image in which texture analysis is performed
	int winsize;
};

#endif