#ifndef _DLLBP_H_
#define _DLLBP_H_

extern "C"
{
	#include "LBP.h"
}
#include <vector>
#include "DLImage.h"

using namespace std;

/**
 * \brief The DLLBP object is a wrapper class for the C implementation of local binary patterns (LBP),
 * a rotation invariant texture analysis approach.
 */
class DLLBP
{
public:

	/**
	* Default DLLBP Constructor.
	*/
	DLLBP();

	/**
	* Default DLLBP Destructor.
	*/
	virtual ~DLLBP();

	/**
	* Compute and return the normalized histogram representation of texture information in type vector<double>
	* \param im input DLImage
	* \param vector<double> The returned normalized histogram representation of texture information
	*/
	vector<double> getNormalizedHistogram( DLImage im );

	/**
	* Compute and return the histogram representation of texture information in type vector<int>
	* \param im input DLImage
	* \param vector<int> The returned histogram bin of texture information
	*/
	vector<int> getHistogram( DLImage im );

	/**
	* Save the histogram representation of texture information in specified file as plain text
	* \param filename output plain text file name in string
	* \param return true if successful
	*/
	bool saveHist( string filename );

protected:

	/// the histogram representation of texture information
	vector<int> hist;
};

#endif